local _G = getfenv(0)
local core
local name, localizedname = GetAddOnInfo("YurrCombatLog")
core = DongleStub("Dongle-1.2"):New(name)
_G.YurrCombatLog = core
local L = _G.YurrCombatLogLocale
if tekDebug then core:EnableDebug(1, tekDebug:GetFrame("YurrCombatLog-Log")) end

local db
local schools = {
	[1] = SPELL_SCHOOL0_NAME, -- physical
	[2] = SPELL_SCHOOL1_NAME, -- holy
	[4] = SPELL_SCHOOL2_NAME, -- fire
	[8] = SPELL_SCHOOL3_NAME, -- nature
	[16] = SPELL_SCHOOL4_NAME, -- frost
	[20] = SPELL_SCHOOL4_NAME..SPELL_SCHOOL2_NAME, -- frostfire
	[32] = SPELL_SCHOOL5_NAME, -- shadow
	[40] = SPELL_SCHOOL3_NAME..SPELL_SCHOOL5_NAME, -- natureshadow
	[64] = SPELL_SCHOOL6_NAME -- arcane
}

function core:Initialize()
	local cmd = self:InitializeSlashCommand("YurrCombatLog slash commands", "yurrcombatlog", "ycl")
	cmd:RegisterSlashHandler("Open the options menu", "^$", "showoptionsgui")

	if YurrCombatLogDB.namespaces then YurrCombatLogDB = nil end
    self.db = self:InitializeDB("YurrCombatLogDB", {
		global = {
			outputFrame = "ChatFrame3",
			unitColors = {
				[1297] = "8080FF", -- self
				[4991] = "804D26", -- pet
				[1394] = "A6FFA6", -- party
				[1396] = "FF7038", -- raid
				[9087] = "404080", -- guardian
				[131072] = "CC54D9", -- focus
				[20334] = "FF75FF", -- enemy players/npcs
				[3864] = "FFFFFF", -- outsider
				--[[[1297] = "8080FF", -- self
				[4369] = "804D26", -- pet
				[1298] = "A6FFA6", -- party
				[1300] = "FF7038", -- raid
				[9087] = "404080", -- guardian
				[131072] = "CC54D9", -- focus
				[20334] = "FF75FF", -- enemy players/npcs
				[1304] = "FFFFFF", -- outsider]]
			},
			spellColors = {
				[1] = "FFFFFF", -- physical
				[2] = "FFFF4D", -- holy
				[4] = "FF262E", -- fire
				[8] = "66FF66", -- fire
				[16] = "4D4DE6", -- frost
				[20] = "A63B96", -- frostfire
				[32] = "FFB3FF", -- shadow
				[40] = "B3D9B3", -- natureshadow
				[64] = "BFBFBF", -- arcane
			},
			otherColors = {
				heal = "FFFF4D",
				miss = "FF8080", -- unused for the moment
				buff = "33FF33",
				debuff = "FF3333",
				skill = "FFFF66",
				timestampColor = "737373",
			},
			braceSrcUnit = true,
			leftSrcBrace = "[",
			rightSrcBrace = "]",
			colorSrc = true,
			colorSrcBraces = false,
			leftSrcBraceColor = "FF75FF",
			rightSrcBraceColor = "FF75FF",
			
			prefix = true,
			colorPrefix = true,
			["++"] = "++",
			["--"] = "--",
			["-+"] = "-+",
			
			braceSpell = true,
			leftSpellBrace = "[",
			rightSpellBrace = "]",
			colorSpell = true,
			colorSpellBraces = false,
			leftSpellBraceColor = "FF75FF",
			rightSpellBraceColor = "FF75FF",
			
			spellSchools = false,
			swingSchools = false,
			rangedSchools = false,
			
			braceDestUnit = true,
			leftDestBrace = "[",
			rightDestBrace = "]",
			colorDest = true,
			colorDestBraces = false,
			leftDestBraceColor = "FF75FF",
			rightDestBraceColor = "FF75FF",
			
			braceAmount = true,
			leftAmountBrace = "[",
			rightAmountBrace = "]",
			colorAmount = true,
			colorAmountBraces = false,
			leftAmountBraceColor = "FF75FF",
			rightAmountBraceColor = "FF75FF",
			braceAmountDots = true,
			leftAmountDotsBrace = "~",
			rightAmountDotsBrace = "~",
			colorAmountDotBraces = false,
			leftAmountDotBraceColor = "FF75FF",
			rightAmountDotBraceColor = "FF75FF",
			markCrits = true,
			leftCrit = "*",
			rightCrit = "*",
			
			overhealing = true,
			overkill = true,
			
			schoolText = false,
			colorSchoolText = true,
			braceSchoolText = false,
			colorSchoolTextBraces = false,
			schoolTextBraceLeft = "[",
			schoolTextBraceRight = "]",
			
			format = {
				ABSORB = "(A%d)",
				BLOCK = BLOCK,
				BLOCK_VALUE = "(B%d)",
				CRITICAL = L["Crit"],
				CRUSHING = L["Crush"],
				DEFLECT = DEFLECT,
				DODGE = DODGE,
				EVADE = L["Evade"],
				GLANCING = L["Glancing"],
				IMMUNE = SPELL_FAILED_IMMUNE,
				MISS = MISS,
				PARRY = PARRY,
				REFLECT = REFLECT,
				RESIST = "(R%d)",
				VULNERABILITY = "(V%d+)",
			},
		},
	}, "global")
    db = self.db[self.db:GetCurrentProfile()]
end

local function showoptionsgui()
    if not IsAddOnLoaded("TinyTipOptions") then
        local _, reason = LoadAddOn("TinyTipOptions")
        local _, title = GetAddOnInfo("TinyTipOptions")
        if (not title) then title = 'TinyTipOptions' end
        if reason then
            core:Print( title .. " (Enable) LoadOnDemand Error - " .. reason )
        else
            --self:Print( "Loaded " .. title)
        end
    end
    if (TinyTipOptions) then TinyTipOptions:Show() end
end

function core:Enable()
	core:RegisterEvent("COMBAT_LOG_EVENT")
	
	local enabled, loadable = select(4, GetAddOnInfo("YurrCombatLogOptions"))
	if enabled and loadable then
		local hijack = CreateFrame("Frame", nil, InterfaceOptionsFrame)
		hijack:SetScript("OnShow", function()
			hijack:Hide()
			local _, reason = LoadAddOn("YurrCombatLogOptions")
			local _, title = GetAddOnInfo("YurrCombatLogOptions")
			if (not title) then title = 'YurrCombatLogOptions' end
			if reason then
				self:Print( title .. " (Enable) LoadOnDemand Error - " .. reason )
			end
		end)
	end
    _G.SlashCmdList["ycl"] = showoptionsgui
end

function core:AffiliationColor(flags)
	local color
	for mask in pairs( db.unitColors ) do
		if ( CombatLog_Object_IsA (flags, mask) )then
			color = db.unitColors[mask]
			break
		end
	end
	return color
end

function core:ColorSpellBySchool(school, aura, event)
	if event == "SPELL_AURA_REMOVED" or event == "SPELL_AURA_REMOVED_DOSE" then
		return db.otherColors.skill
	elseif aura == "BUFF" then
		return db.otherColors.buff
	elseif aura =="DEBUFF" then
		return db.otherColors.debuff
	elseif event == "SPELL_HEAL" or event == "SPELL_PERIODIC_HEAL" then
		return db.otherColors.heal
	else
		if not db.spellColors[school] then self:Debug(1, school) end
		return db.spellColors[school]-- or "FFFFFF"
	end
end

function core:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing)

	local resultStr
	
	if ( resisted or blocked or absorbed or critical or glancing or crushing or useOverhealing or useOverkill) then
		
		if ( resisted ) then
			if ( resisted < 0 ) then	--Its really a vulnerability
				resultStr = format(db.format.VULNERABILITY, -resisted);
			else
				resultStr = format(db.format.RESIST, resisted);
			end
		end
		if ( blocked ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(db.format.BLOCK_VALUE, blocked);
			else
				resultStr = format(db.format.BLOCK_VALUE, blocked);
			end
		end
		if ( absorbed ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..format(db.format.ABSORB, absorbed);
			else
				resultStr = format(db.format.ABSORB, absorbed);
			end
		end
		if ( glancing ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..db.format.GLANCING;
			else
				resultStr = db.format.GLANCING;
			end
		end
		if ( crushing ) then
			if ( resultStr ) then
				resultStr = resultStr.." "..db.format.CRUSHING;
			else
				resultStr = db.format.CRUSHING;
			end
		end
		if ( critical ) then
			local critString = db.format.CRITICAL;
			if ( spellId ) then
				critString = db.format.CRITICAL_SPELL;
			end
			if ( resultStr ) then
				resultStr = resultStr.." "..critString;
			else
				resultStr = critString;
			end
		end
	end
	
	return resultStr or ""

end

function core:COMBAT_LOG_EVENT(...)
	self:Debug(1, ...)
	
	local event = select(3, ...)
	
	local srcUnitName = select(5, ...)
	local srcFlags = select(6, ...)
	local srcUnitNameString
	
	local prefixString
	
	local spellName = select(11, ...)
	local school = select(12, ...)
	local spellNameString
	
	local suffixString
	
	local destUnitName = select(8, ...)
	local destFlags = select(9, ...)
	local destUnitNameString
	
	local amount
	if event == "SWING_DAMAGE" then
		amount = select(10, ...)
	else
		amount = select(13, ...)
	end
	local amountString
	local schoolStr
	local resultString
	
	local finalString
	
	if srcUnitName == UnitName("player") then srcUnitName = UNIT_YOU end
	if destUnitName == UnitName("player") then destUnitName = UNIT_YOU end
	
	if event == "ENVIRONMENTAL_DAMAGE" then
		srcUnitName = destUnitName
		srcFlags = destFlags
		destUnitName = nil
		destFlags = nil
		spellName = L[select(10, ...)]
		amount = select(11, ...)
		school = select(13, ...)
	elseif event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED" then
		spellName = select(10, ...)
		amount = select(12, ...)
	elseif event == "SWING_MISSED" then
		local tmpSrcUnitName = srcUnitName
		local tmpSrcFlags = srcFlags
		local tmpDestUnitName = destUnitName
		local tmpDestFlags = destFlags
		local miss = select(10, ...)
		if not miss == "MISS" then
			srcUnitName = tmpDestUnitName
			srcFlags = tmpDestFlags
			destUnitName = tmpSrcUnitName
			destFlags = tmpSrcFlags
		end
		
		spellName = db.format[miss]
		if spellName == db.format["BLOCK"] then
			amount = select(11, ...)
			school = 1
		end
	elseif event == "SPELL_AURA_APPLIED_DOSE" or event == "SPELL_AURA_REMOVED_DOSE" then
		local count = select(14, ...)
		spellName = spellName .. "(" .. count .. ")"
	end
	
	if srcUnitName then
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if db.colorSrc then
			local color = self:AffiliationColor(srcFlags)
			coloredUnitStr = "|cFF" .. color .. srcUnitName .. "|r"
		end
		if db.colorSrcBraces then
			leftBrace = "|cFF" .. db.leftSrcBraceColor .. db.leftSrcBrace .. "|r"
			rightBrace = "|cFF" .. db.rightSrcBraceColor .. db.rightSrcBrace .. "|r"
		else
			leftBrace = db.leftSrcBrace
			rightBrace = db.rightSrcBrace
		end
		if db.braceSrcUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		srcUnitNameString = format(str or "%s ", coloredUnitStr or srcUnitName)
	else
		srcUnitNameString = ""
	end
	if event == "UNIT_DIED" then
		srcUnitNameString = TUTORIAL_TITLE25 .. ": " .. srcUnitNameString
	elseif event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED" then
		srcUnitNameString = ENSCRIBE .. ": " .. srcUnitNameString
	end
	
	if db.prefix then
		local str
		local aura = select(13, ...)
		if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
			str = db["++"]
		elseif event == "SPELL_AURA_REMOVED" or event == "SPELL_AURA_REMOVED_DOSE" then
			str = db["--"]
		elseif event == "SPELL_AURA_REFRESH" then
			str = db["-+"]
		elseif event == "SPELL_CAST_START" then
			str = L["begins"]
		elseif event == "SPELL_CAST_SUCCESS" then
			str = L["casts"]
		else
			str = ""
		end
		if db.colorPrefix and (str ~= "") then
			local color = self:ColorSpellBySchool(school, aura, event) or "FFFFFF"
			prefixString = format("|cFF%s%s|r ", color, str)
		elseif (str ~= "") then
			prefixString = str .. " "
		else
			prefixString = str
		end
	else
		prefixString = ""
	end
	
	if event == "PARTY_KILL" then
		spellName = ACTION_PARTY_KILL
	elseif event == "SWING_DAMAGE" then
		spellName = MELEE
	end
	if spellName then
		local str
		local coloredSpellStr
		local leftBrace
		local rightBrace
		if db.colorSpell then
			local aura = select(13, ...)
			local color = self:ColorSpellBySchool(school, aura, event) or "FFFFFF"
			coloredSpellStr = "|cFF" .. color .. spellName .. "|r"
		end
		if db.colorSpellBraces then
			leftBrace = "|cFF" .. db.leftSpellBraceColor .. db.leftSpellBrace .. "|r"
			rightBrace = "|cFF" .. db.rightSpellBraceColor .. db.rightSpellBrace .. "|r"
		end
		if db.braceSpell then
			str = (leftBrace or "") .. "%s" .. (rightBrace or "") .. " "
		end
		spellNameString = format(str or "%s ", coloredSpellStr or spellName)
	else
		spellNameString = ""
	end
	
	if db.suffixString then
		
	end
	
	if destUnitName then
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if db.colorDest then
			local color = self:AffiliationColor(destFlags)
			coloredUnitStr = "|cFF" .. color .. destUnitName .. "|r"
		end
		if db.colorDestBraces then
			leftBrace = "|cFF" .. db.leftDestBraceColor .. db.leftDestBrace .. "|r"
			rightBrace = "|cFF" .. db.rightDestBraceColor .. db.rightDestBrace .. "|r"
		else
			leftBrace = db.leftDestBrace
			rightBrace = db.rightDestBrace
		end
		if db.braceDestUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		destUnitNameString = format(str or "%s ", coloredUnitStr or destUnitName)
	else
		destUnitNameString = ""
	end
	
	if type(amount) == "number" then
		local str
		local amountResult
		local coloredAmountStr
		local leftBrace
		local rightBrace
		local leftDot
		local rightDot
		local crit = select(19, ...)
		local leftCrit
		local rightCrit
		local finalStr
		if event == "SWING_DAMAGE" then
			crit = select(15, ...)
		elseif event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "RANGED_DAMAGE" or event == "ENVIRONMENTAL_DAMAGE" then
			crit = select(19, ...)
		end
		if db.overhealing then
			local overhealing = select(14, ...)
			if type(overhealing) ==  "number" and event == "SPELL_HEAL" or event == "SPELL_PERIODIC_HEAL" then
				if overhealing > 0 then
					amountResult = amount - overhealing .. "(+" .. overhealing .. ")"
				end
			end
		end
		if db.overkill then
			local overkill
			if event == "SWING_DAMAGE" then
				overkill = select(11, ...)
			elseif event == "SPELL_PERIODIC_DAMAGE" then
				overkill = select(14, ...)
			elseif event == "SPELL_DAMAGE" or event == "RANGED_DAMAGE" then
				overkill = select(14, ...)
			end
			if type(overkill) == "number" then
				if overkill > 0 then
					amountResult = amount - overkill .. "(+" .. overkill .. ")"
				end
			end
		end
		if not amountResult then amountResult = amount end
		if event == "SPELL_ENERGIZE" or event == "SPELL_PERIODIC_ENERGIZE" then
			amountResult = "+ " .. amountResult
		end
		if db.colorAmount then
			local aura = select(13, ...)
			local color = self:ColorSpellBySchool(school, aura, event)
			coloredAmountStr = "|cFF" .. color .. amountResult .. "|r"
		end
		if db.braceAmount then
			if db.colorAmountBraces then
				leftBrace = "|cFF" .. db.leftAmountBraceColor .. db.leftAmountBrace .. "|r"
				rightBrace = "|cFF" .. db.rightAmountBraceColor .. db.rightAmountBrace .. "|r"
			else
				leftBrace = ""
				rightBrace = ""
			end
			str = "%s%s" .. leftBrace .. "%s" .. rightBrace .. "%s%s" .. " "
		end
		if db.braceAmountDots and event == "SPELL_PERIODIC_DAMAGE" or event == "SPELL_PERIODIC_HEAL" then
			if colorAmountDotBraces then
				leftDot = "|cFF" .. leftAmountDotBraceColor .. db.leftAmountDotsBrace .. rightAmountDotBraceColor .. "|r"
				leftDot = "|cFF" .. leftAmountDotBraceColor .. db.rightAmountDotsBrace .. rightAmountDotBraceColor .. "|r"
			else
				leftDot = db.leftAmountDotsBrace
				rightDot = db.rightAmountDotsBrace
			end
		else
			leftDot = ""
			rightDot = ""
		end
		if db.markCrits and crit then
			leftCrit = db.leftCrit
			rightCrit = db.rightCrit
		else
			leftCrit = ""
			rightCrit = ""
		end
		amountNameString = format(str or "%s%s%s%s%s ", leftCrit, leftDot, coloredAmountStr or amountResult, rightDot, rightCrit)
	elseif amountNameString ~= nil and (event == "ENCHANT_APPLIED" or event == "ENCHANT_REMOVED") then
		amountNameString = amount
	else
		amountNameString = ""
	end
	
	if db.schoolText and (event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "SWING_DAMAGE" or event == "RANGED_DAMAGE" or event == "ENVIRONMENTAL_DAMAGE") then
		local str1
		local str2
		if db.colorSchoolText then
			local color = self:ColorSpellBySchool(school)
			str1 = "|cFF" .. color .. schools[school] .. "|r"
		else
			str1 = schools[school]
		end
		if db.braceSchoolText then
			if db.colorSchoolTextBraces then
				local color = self:ColorSpellBySchool(school)
				str2 = "|cFF" .. color .. db.schoolTextBraceLeft .. "|r" .. str1 .. "|cFF" .. color .. db.schoolTextBraceRight .. "|r"
			else
				str2 = db.schoolTextBraceLeft .. str1 .. db.schoolTextBraceRight
			end
		else
			str2 = str1
		end
		schoolStr = str2
	else
		schoolStr = ""
	end
	
	local resisted, blocked, absorbed, critical, glancing, crushing
	if event == "SWING_DAMAGE" then
		resisted, blocked, absorbed, critical, glancing, crushing = select(13, ...)
	elseif event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "RANGED_DAMAGE" or event == "ENVIRONMENTAL_DAMAGE" then
		resisted, blocked, absorbed, critical, glancing, crushing = select(16, ...)
	end
	resultString = self:DamageResult(resisted, blocked, absorbed, critical, glancing, crushing)
	
	finalString = srcUnitNameString .. prefixString .. spellNameString .. destUnitNameString .. amountNameString .. schoolStr .. resultString
	_G[db.outputFrame]:AddMessage(finalString)
end
